/*
3d.h
Headre file for 3d.c
*/

#define NUM_CLIPPING_PLANES 5	// For clipping to viewing frustum.

typedef struct camera		/* Store information about a camera */
{
	fixed x,y,z;		// Camera's position
	fixed xf,yf,zf;		// Where camera is pointing.
	fixed xup,yup,zup;	// The camera's up position.
	fixed lon;			// Camera's rotation around the X axis (longitude).
	fixed lat;			// Camera's rotation around the Y axis (lattitude).
	fixed spin;			// Camera's spin.
	fixed alt;			// Camera's altitude.
	fixed fov;			// Field of view
	fixed aspect;		// Aspect
	MATRIX camera_matrix;	// The camera's matrix.
} camera;

typedef struct camera_f		/* Store information about a float camera */
{
	float x,y,z;		// Camera's position
	float xf,yf,zf;		// Where camera is pointing.
	float xup,yup,zup;	// The camera's up position.
	float lon;			// Camera's rotation around the X axis (longitude).
	float lat;			// Camera's rotation around the Y axis (lattitude).
	float spin;			// Camera's spin.
	float alt;			// Camera's altitude.
	float fov;			// Field of view
	float aspect;		// Aspect
	MATRIX_f camera_matrix;	// The camera's matrix.
} camera_f;

fixed z_normal_polygon(V3D *vertex1, V3D *vertex2, V3D *vertex3);
float z_normal_polygon_f(V3D_f *vertex1, V3D_f *vertex2, V3D_f *vertex3);

fixed get_latitude(fixed Px, fixed Py, fixed Pz);
float get_latitude_f(float Px, float Py, float Pz);

fixed get_longitude(fixed Px, fixed Py, fixed Pz);
float get_longitude_f(float Px, float Py, float Pz);

void get_polar_positioning_matrix(MATRIX *polar_position_matrix, fixed lat, fixed lon);
void get_polar_positioning_matrix_f(MATRIX_f *polar_position_matrix, float lat, float lon);

void polar_rotate_camera(camera *the_camera, fixed dlat, fixed dlon, fixed dalt, fixed dspin, fixed xmiddle, fixed ymiddle, fixed zmiddle);
void polar_rotate_camera_f(camera_f *the_camera, float dlat, float dlon, float dalt, float dspin, float xmiddle, float ymiddle, float zmiddle);

void reverse_persp_project_f(float x, float y, float z, float *xout, float *yout);

void matrix_to_matrix_f(MATRIX *in_matrix, MATRIX_f *out_matrix_f);
void matrix_f_to_matrix(MATRIX_f *in_matrix_f, MATRIX *out_matrix);

int clip_poly_to_frustum_f(V3D_f *out_vertices_1, V3D_f *out_vertices_2, int vertex_count, float near_z);
