/*

Intern.h

Stuff that should be in Allegro's internal.h

zbuf.c uses it.
Make zbpoly use it too.

*/

#ifndef INTERN_H
#define INTERN_H



struct Z_BUFFER;




/*
some Allegro functions need a block of scratch memory
*/
extern void *_scratch_mem;
extern int _scratch_mem_size;

__INLINE__ void _grow_scratch_mem(int size)
{
   if (size > _scratch_mem_size) {
      size = (size+1023) & 0xFFFFFC00;
      _scratch_mem = realloc(_scratch_mem, size);
      _scratch_mem_size = size;
   }
}


/*
information for polygon scanline fillers
AE ADDED *zb_line
*/
typedef struct POLYGON_SEGMENT
{
   fixed u, v, du, dv;              /* fixed point u/v coordinates */
   fixed c, dc;                     /* single color gouraud shade values */
   fixed r, g, b, dr, dg, db;       /* RGB gouraud shade values */
   float z, dz;                     /* polygon depth (1/z) */
   float fu, fv, dfu, dfv;          /* floating point u/v coordinates */
   unsigned char *texture;          /* the texture map */
   int umask, vmask, vshift;        /* texture map size information */
   int seg;                         /* destination bitmap selector */
   float *zb_line;			/* the start address of the line of the Z buffer */
} POLYGON_SEGMENT;



/*
polygon scanline filler functions
AE ADDED the _zb prefix, and tex_mask_lit
*/
void _zb_poly_scanline_flat(unsigned long addr, int w, POLYGON_SEGMENT *info);
void _zb_poly_scanline_gcol(unsigned long addr, int w, POLYGON_SEGMENT *info);
void _zb_poly_scanline_grgb(unsigned long addr, int w, POLYGON_SEGMENT *info);
void _zb_poly_scanline_atex(unsigned long addr, int w, POLYGON_SEGMENT *info);
void _zb_poly_scanline_ptex(unsigned long addr, int w, POLYGON_SEGMENT *info);
void _zb_poly_scanline_atex_mask(unsigned long addr, int w, POLYGON_SEGMENT *info);
void _zb_poly_scanline_ptex_mask(unsigned long addr, int w, POLYGON_SEGMENT *info);
void _zb_poly_scanline_atex_lit(unsigned long addr, int w, POLYGON_SEGMENT *info);
void _zb_poly_scanline_ptex_lit(unsigned long addr, int w, POLYGON_SEGMENT *info);
void _zb_poly_scanline_atex_mask_lit(unsigned long addr, int w, POLYGON_SEGMENT *info);
void _zb_poly_scanline_ptex_mask_lit(unsigned long addr, int w, POLYGON_SEGMENT *info);






/*
ASM helpers for the zprite routines.
BY AE.
*/

void _zprite_make_z(BITMAP *the_sprite, struct Z_BUFFER *the_zbuffer, float z, int colour);

void _linear_draw_zprite(BITMAP *destbmp, struct Z_BUFFER *dest_zbuffer, BITMAP *the_sprite, struct Z_BUFFER *the_zbuffer, int xpos, int ypos);
void _linear_draw_zprite_v_flip(BITMAP *destbmp, struct Z_BUFFER *dest_zbuffer, BITMAP *the_sprite, struct Z_BUFFER *the_zbuffer, int xpos, int ypos);
void _linear_draw_zprite_h_flip(BITMAP *destbmp, struct Z_BUFFER *dest_zbuffer, BITMAP *the_sprite, struct Z_BUFFER *the_zbuffer, int xpos, int ypos);
void _linear_draw_zprite_vh_flip(BITMAP *destbmp, struct Z_BUFFER *dest_zbuffer, BITMAP *the_sprite, struct Z_BUFFER *the_zbuffer, int xpos, int ypos);
void _linear_draw_trans_zprite(BITMAP *destbmp, struct Z_BUFFER *dest_zbuffer, BITMAP *the_sprite, struct Z_BUFFER *the_zbuffer, int xpos, int ypos);
void _linear_draw_lit_zprite(BITMAP *destbmp, struct Z_BUFFER *dest_zbuffer, BITMAP *the_sprite, struct Z_BUFFER *the_zbuffer, int xpos, int ypos, int color);

void _linear_draw_flat_zprite(BITMAP *destbmp, struct Z_BUFFER *dest_zbuffer, BITMAP *the_sprite, int xpos, int ypos, float z);
void _linear_draw_flat_zprite_h_flip(BITMAP *destbmp, struct Z_BUFFER *dest_zbuffer, BITMAP *the_sprite, int xpos, int ypos, float z);
void _linear_draw_flat_zprite_v_flip(BITMAP *destbmp, struct Z_BUFFER *dest_zbuffer, BITMAP *the_sprite, int xpos, int ypos, float z);
void _linear_draw_flat_zprite_vh_flip(BITMAP *destbmp, struct Z_BUFFER *dest_zbuffer, BITMAP *the_sprite, int xpos, int ypos, float z);
void _linear_draw_flat_trans_zprite(BITMAP *destbmp, struct Z_BUFFER *dest_zbuffer, BITMAP *the_sprite, int xpos, int ypos, float z);
void _linear_draw_flat_lit_zprite(BITMAP *destbmp, struct Z_BUFFER *dest_zbuffer, BITMAP *the_sprite, int xpos, int ypos, float z, int color);

/* asm helper for zb_stretch_blit() */
void _do_zb_displace(BITMAP *source, BITMAP *dest, void *drawer, int sx, fixed sy, fixed syd, int dx, int dy, int dh, struct Z_BUFFER *source_zbuffer, struct Z_BUFFER *dest_zbuffer, float z);
void _do_flat_zb_displace(BITMAP *source, BITMAP *dest, void *drawer, int sx, fixed sy, fixed syd, int dx, int dy, int dh, struct Z_BUFFER *dest_zbuffer);


#endif          /* ifndef INTERN_H */
