/*
 *      By Shawn Hargreaves,
 *
 *      i386 opcodes for the bitmap stretcher and mode-X sprite compilers.
 *
 */

/*
Modified by Andrei Ellman to add aditional opcodes for doing z-buffered stretching and stretching with transparency.
*/


#ifndef OPCODES_H
#define OPCODES_H

#include "intern.h"


#define COMPILER_BYTE(val) {                                                 \
   *(((char *)_scratch_mem)+compiler_pos) = val;                             \
   compiler_pos++;                                                           \
}


#define COMPILER_WORD(val) {                                                 \
   *((short *)(((char *)_scratch_mem)+compiler_pos)) = val;                  \
   compiler_pos += 2;                                                        \
}


#define COMPILER_LONG(val) {                                                 \
   *((long *)(((char *)_scratch_mem)+compiler_pos)) = val;                   \
   compiler_pos += 4;                                                        \
}

/* AE */
#define COMPILER_FLOAT(val) {                                                 \
   *((float *)(((char *)_scratch_mem)+compiler_pos)) = val;                   \
   compiler_pos += 4;                                                        \
}

/* AE */
#define COMPILER_PTR(val) {                                                 \
   *((void **)(((char *)_scratch_mem)+compiler_pos)) = val;                   \
   compiler_pos += 4;                                                        \
}


#define COMPILER_INC_ESI() {                                                 \
   _grow_scratch_mem(compiler_pos+1);                                        \
   COMPILER_BYTE(0x46);          /* incl %esi */                             \
}


#define COMPILER_INC_EDI() {                                                 \
   _grow_scratch_mem(compiler_pos+1);                                        \
   COMPILER_BYTE(0x47);          /* incl %edi */                             \
}


/* AE */
#define COMPILER_ADD_EBX(val) {                                              \
   _grow_scratch_mem(compiler_pos+6);                                        \
   COMPILER_BYTE(0x81);          /* addl $val, %ebx */                       \
   COMPILER_BYTE(0xC3);                                                      \
   COMPILER_LONG(val);                                                       \
}


/* AE */
#define COMPILER_ADD_EBX_4() {                                              \
   _grow_scratch_mem(compiler_pos+6);                                        \
   COMPILER_BYTE(0x83);          /* addl $4, %ebx (adds a one-byte value) */ \
   COMPILER_BYTE(0xC3);                                                      \
   COMPILER_BYTE(0x04);                                                      \
}


/* AE */
#define COMPILER_ADD_ECX(val) {                                              \
   _grow_scratch_mem(compiler_pos+6);                                        \
   COMPILER_BYTE(0x81);          /* addl $val, %ecx */                       \
   COMPILER_BYTE(0xC1);                                                      \
   COMPILER_LONG(val);                                                       \
}


/* AE */
#define COMPILER_ADD_ECX_4() {                                              \
   _grow_scratch_mem(compiler_pos+6);                                        \
   COMPILER_BYTE(0x83);          /* addl $4, %ecx (adds a one-byte value) */ \
   COMPILER_BYTE(0xC1);                                                      \
   COMPILER_BYTE(0x04);                                                      \
}


#define COMPILER_ADD_ESI(val) {                                              \
   _grow_scratch_mem(compiler_pos+6);                                        \
   COMPILER_BYTE(0x81);          /* addl $val, %esi */                       \
   COMPILER_BYTE(0xC6);                                                      \
   COMPILER_LONG(val);                                                       \
}


#define COMPILER_ADD_ECX_EAX() {                                             \
   _grow_scratch_mem(compiler_pos+2);                                        \
   COMPILER_BYTE(0x01);          /* addl %ecx, %eax */                       \
   COMPILER_BYTE(0xC8);                                                      \
}


#define COMPILER_MOV_EAX(val) {                                              \
   _grow_scratch_mem(compiler_pos+5);                                        \
   COMPILER_BYTE(0xB8);          /* movl $val, %eax */                       \
   COMPILER_LONG(val);                                                       \
}


#define COMPILER_MOV_ECX(val) {                                              \
   _grow_scratch_mem(compiler_pos+5);                                        \
   COMPILER_BYTE(0xB9);          /* movl $val, %ecx */                       \
   COMPILER_LONG(val);                                                       \
}


#define COMPILER_MOV_EDX(val) {                                              \
   _grow_scratch_mem(compiler_pos+5);                                        \
   COMPILER_BYTE(0xBA);          /* movl $val, %edx */                       \
   COMPILER_LONG(val);                                                       \
}


#define COMPILER_MOV_EDI_EAX() {                                             \
   _grow_scratch_mem(compiler_pos+2);                                        \
   COMPILER_BYTE(0x89);          /* movl %edi, %eax */                       \
   COMPILER_BYTE(0xF8);                                                      \
}


#define COMPILER_CALL_ESI() {                                                \
   _grow_scratch_mem(compiler_pos+2);                                        \
   COMPILER_BYTE(0xFF);          /* call *%esi */                            \
   COMPILER_BYTE(0xD6);                                                      \
}


#define COMPILER_OUTW() {                                                    \
   _grow_scratch_mem(compiler_pos+2);                                        \
   COMPILER_BYTE(0x66);          /* outw %ax, %dx */                         \
   COMPILER_BYTE(0xEF);                                                      \
}


#define COMPILER_PUSH_ESI() {                                                \
   _grow_scratch_mem(compiler_pos+1);                                        \
   COMPILER_BYTE(0x56);          /* pushl %esi */                            \
}


#define COMPILER_PUSH_EDI() {                                                \
   _grow_scratch_mem(compiler_pos+1);                                        \
   COMPILER_BYTE(0x57);          /* pushl %edi */                            \
}


#define COMPILER_POP_ESI() {                                                 \
   _grow_scratch_mem(compiler_pos+1);                                        \
   COMPILER_BYTE(0x5E);          /* popl %esi */                             \
}


#define COMPILER_POP_EDI() {                                                 \
   _grow_scratch_mem(compiler_pos+1);                                        \
   COMPILER_BYTE(0x5F);          /* popl %edi */                             \
}


#define COMPILER_REP_MOVSB() {                                               \
   _grow_scratch_mem(compiler_pos+2);                                        \
   COMPILER_BYTE(0xF2);          /* rep */                                   \
   COMPILER_BYTE(0xA4);          /* movsb */                                 \
}

/* Modified by AE */
#define COMPILER_LODSB() {                                                   \
   _grow_scratch_mem(compiler_pos+3);                                        \
   COMPILER_BYTE(0x8A);          /* movb (%esi), %ah */                      \
   COMPILER_BYTE(0x26);                                                      \
   COMPILER_BYTE(0x46);          /* incl %esi */                             \
}

/* AE */
// This is just regular lodsb.
#define COMPILER_ZB_LODSB() {                                                \
   _grow_scratch_mem(compiler_pos+3);                                       \
   COMPILER_BYTE(0x8A);          /* movb (%esi), %ah */                      \
   COMPILER_BYTE(0x26);                                                      \
   COMPILER_BYTE(0x46);          /* incl %esi */                             \
}

/* AE */
#define COMPILER_ZB_START_NEXT_CALC() {                                      \
   _grow_scratch_mem(compiler_pos+15);                                       \
   COMPILER_BYTE(0x83);        /* addl $4, %ebx */  /* Advance source ZB */  \
   COMPILER_BYTE(0xC3);                                                      \
   COMPILER_BYTE(0x04);                                                      \
   COMPILER_BYTE(0xD9);        /* fld %st	*/	            /* D D */        \
   COMPILER_BYTE(0xC0);                                                      \
   COMPILER_BYTE(0xD9);        /* flds (%ebx) */          /* K D D */        \
   COMPILER_BYTE(0x03);                                                      \
   COMPILER_BYTE(0xDC);        /* fmul %st,%st(1) */      /* K DK D */       \
   COMPILER_BYTE(0xC9);                                                      \
   COMPILER_BYTE(0xD9);        /* fld1 */               /* 1 K DK D */       \
   COMPILER_BYTE(0xE8);                                                      \
   COMPILER_BYTE(0xDE);        /* faddp %st,%st(2) */   /* K 1+DK D */       \
   COMPILER_BYTE(0xC2);                                                      \
   COMPILER_BYTE(0xDE);        /* fdivp %st,%st(1) */   /* K/1+DK D */       \
   COMPILER_BYTE(0xF1);                                                      \
}

/* AE */
#define COMPILER_ZB_END_THIS_CALC() {                                        \
   _grow_scratch_mem(compiler_pos+12);                                       \
   COMPILER_BYTE(0xD9);          /* fstps &tmp */         /* D */            \
   COMPILER_BYTE(0x1D);                                                      \
   COMPILER_PTR(&tmp);                                                       \
   COMPILER_BYTE(0x8B);          /* movl &tmp, %edx */                       \
   COMPILER_BYTE(0x15);                                                      \
   COMPILER_PTR(&tmp);                                                       \
}


/* Modified by AE */
#define COMPILER_STOSB() {                                                   \
   _grow_scratch_mem(compiler_pos+4);                                        \
   COMPILER_BYTE(0x26);          /* movb %ah, %es:(%edi) */                  \
   COMPILER_BYTE(0x88);                                                      \
   COMPILER_BYTE(0x27);                                                      \
   COMPILER_BYTE(0x47);          /* incl %edi */                             \
}

/* Modified by AE */
#define COMPILER_MASKED_STOSB() {                                            \
   _grow_scratch_mem(compiler_pos+8);                                        \
   COMPILER_BYTE(0x08);          /* orb %ah, %ah */                          \
   COMPILER_BYTE(0xE4);                                                      \
   COMPILER_BYTE(0x74);          /* jz skip */                               \
   COMPILER_BYTE(0x03);                                                      \
   COMPILER_BYTE(0x26);          /* movb %ah, %es:(%edi) */                  \
   COMPILER_BYTE(0x88);                                                      \
   COMPILER_BYTE(0x27);                                                      \
   COMPILER_BYTE(0x47);          /* skip: incl %edi */                       \
}


/* AE */
#define COMPILER_ZB_STOSB() {                                                \
   _grow_scratch_mem(compiler_pos+13);                                       \
   COMPILER_BYTE(0x39);          /* cmpl %edx, (%ecx) */                     \
   COMPILER_BYTE(0x11);                                                      \
   COMPILER_BYTE(0x7D);          /* jge skip */                              \
   COMPILER_BYTE(0x05);                                                      \
   COMPILER_BYTE(0x89);          /* movl %edx, (%ecx) */                     \
   COMPILER_BYTE(0x11);                                                      \
   COMPILER_BYTE(0x26);          /* movb %ah, %es:(%edi) */                  \
   COMPILER_BYTE(0x88);                                                      \
   COMPILER_BYTE(0x27);                                                      \
   COMPILER_BYTE(0x47);          /* skip: incl %edi */                       \
   COMPILER_BYTE(0x83);          /* addl $4, %ecx */                         \
   COMPILER_BYTE(0xC1);                                                      \
   COMPILER_BYTE(0x04);                                                      \
}


/* AE */
#define COMPILER_TRANS_ZB_STOSB() {                                          \
   _grow_scratch_mem(compiler_pos+22);                                       \
   COMPILER_BYTE(0x39);          /* cmpl %edx, (%ecx) */                     \
   COMPILER_BYTE(0x11);                                                      \
   COMPILER_BYTE(0x7D);          /* jge skip */                              \
   COMPILER_BYTE(0x0E);                                                      \
   COMPILER_BYTE(0x89);          /* movl %edx, (%ecx) */                     \
   COMPILER_BYTE(0x11);                                                      \
   COMPILER_BYTE(0x26);          /* movb %es:(%edi), %al */                  \
   COMPILER_BYTE(0x8A);                                                      \
   COMPILER_BYTE(0x07);                                                      \
   COMPILER_BYTE(0x8A);          /* movb color_map(%eax), al */              \
   COMPILER_BYTE(0x80);                                                      \
   COMPILER_PTR(color_map);                                                  \
   COMPILER_BYTE(0x26);          /* movb %al, %es:(%edi) */                  \
   COMPILER_BYTE(0x88);                                                      \
   COMPILER_BYTE(0x07);                                                      \
   COMPILER_BYTE(0x47);          /* skip: incl %edi */                       \
   COMPILER_BYTE(0x83);          /* addl $4, %ecx */                         \
   COMPILER_BYTE(0xC1);                                                      \
   COMPILER_BYTE(0x04);                                                      \
}


/* AE */
#define COMPILER_MASKED_FLAT_ZB_STOSB(zvalue) {                                            \
   _grow_scratch_mem(compiler_pos+25);                                       \
   COMPILER_BYTE(0x08);          /* orb %ah, %ah */                          \
   COMPILER_BYTE(0xE4);                                                      \
   COMPILER_BYTE(0x74);          /* jz skip */                               \
   COMPILER_BYTE(0x11);                                                      \
   COMPILER_BYTE(0x81);          /* cmpl zvalue, (%ecx) */                   \
   COMPILER_BYTE(0x39);                                                      \
   COMPILER_FLOAT(zvalue);                                                   \
   COMPILER_BYTE(0x7D);          /* jge skip */                              \
   COMPILER_BYTE(0x09);                                                      \
   COMPILER_BYTE(0xC7);          /* movl zvalue, (%ecx) */                   \
   COMPILER_BYTE(0x01);                                                      \
   COMPILER_FLOAT(zvalue);                                                   \
   COMPILER_BYTE(0x26);          /* movb %ah, %es:(%edi) */                  \
   COMPILER_BYTE(0x88);                                                      \
   COMPILER_BYTE(0x27);                                                      \
   COMPILER_BYTE(0x47);          /* skip: incl %edi */                       \
   COMPILER_BYTE(0x83);          /* addl $4, %ecx */                         \
   COMPILER_BYTE(0xC1);                                                      \
   COMPILER_BYTE(0x04);                                                      \
}


/* AE */
#define COMPILER_MASKED_FLAT_TRANS_ZB_STOSB(zvalue) {                                            \
   _grow_scratch_mem(compiler_pos+34);                                       \
   COMPILER_BYTE(0x08);          /* orb %ah, %ah */                          \
   COMPILER_BYTE(0xE4);                                                      \
   COMPILER_BYTE(0x74);          /* jz skip */                               \
   COMPILER_BYTE(0x1A);                                                      \
   COMPILER_BYTE(0x81);          /* cmpl zvalue, (%ecx) */                   \
   COMPILER_BYTE(0x39);                                                      \
   COMPILER_FLOAT(zvalue);                                                   \
   COMPILER_BYTE(0x7D);          /* jge skip */                              \
   COMPILER_BYTE(0x12);                                                      \
   COMPILER_BYTE(0xC7);          /* movl zvalue, (%ecx) */                   \
   COMPILER_BYTE(0x01);                                                      \
   COMPILER_FLOAT(zvalue);                                                   \
   COMPILER_BYTE(0x26);          /* movb %es:(%edi), %al */                  \
   COMPILER_BYTE(0x8A);                                                      \
   COMPILER_BYTE(0x07);                                                      \
   COMPILER_BYTE(0x8A);          /* movb color_map(%eax), al */              \
   COMPILER_BYTE(0x80);                                                      \
   COMPILER_PTR(color_map);                                                  \
   COMPILER_BYTE(0x26);          /* movb %al, %es:(%edi) */                  \
   COMPILER_BYTE(0x88);                                                      \
   COMPILER_BYTE(0x07);                                                      \
   COMPILER_BYTE(0x47);          /* skip: incl %edi */                       \
   COMPILER_BYTE(0x83);          /* addl $4, %ecx */                         \
   COMPILER_BYTE(0xC1);                                                      \
   COMPILER_BYTE(0x04);                                                      \
}


#define MOV_IMMED_SIZE(x, offset)                                            \
   (compiler_pos + x + (offset < 128) ? 2 : 5)


#define MOV_IMMED(offset) {                                                  \
   if (offset < 128) {                                                       \
      COMPILER_BYTE(0x40);       /* byte address offset */                   \
      COMPILER_BYTE(offset);                                                 \
   }                                                                         \
   else {                                                                    \
      COMPILER_BYTE(0x80);       /* long address offset */                   \
      COMPILER_LONG(offset);                                                 \
   }                                                                         \
}


#define COMPILER_MOVB_IMMED(offset, val) {                                   \
   _grow_scratch_mem(MOV_IMMED_SIZE(3, offset));                             \
   COMPILER_BYTE(0x64);          /* fs: */                                   \
   COMPILER_BYTE(0xC6);          /* movb $val, offset(%eax) */               \
   MOV_IMMED(offset);                                                        \
   COMPILER_BYTE(val);                                                       \
}


#define COMPILER_MOVW_IMMED(offset, val) {                                   \
   _grow_scratch_mem(MOV_IMMED_SIZE(5, offset));                             \
   COMPILER_BYTE(0x66);          /* word prefix */                           \
   COMPILER_BYTE(0x64);          /* fs: */                                   \
   COMPILER_BYTE(0xC7);          /* movw $val, offset(%eax) */               \
   MOV_IMMED(offset);                                                        \
   COMPILER_WORD(val);                                                       \
}


#define COMPILER_MOVL_IMMED(offset, val) {                                   \
   _grow_scratch_mem(MOV_IMMED_SIZE(6, offset));                             \
   COMPILER_BYTE(0x64);          /* fs: */                                   \
   COMPILER_BYTE(0xC7);          /* movl $val, offset(%eax) */               \
   MOV_IMMED(offset);                                                        \
   COMPILER_LONG(val);                                                       \
}


#define COMPILER_RET() {                                                     \
   _grow_scratch_mem(compiler_pos+1);                                        \
   COMPILER_BYTE(0xC3);          /* ret */                                   \
}


#endif          /* ifndef OPCODES_H */

